@@ -1,7 +1,7 @@
 from abc import ABCMeta, abstractmethod
 from enum import Enum
 from operator import attrgetter
-from typing import Any, Dict, Tuple, TypeVar
+from typing import Any, Dict, Set, Tuple, TypeVar
 
 from .span import AbstractSpan, AbstractTalismanSpan
 from .tree import AbstractTree
@@ -47,6 +47,23 @@ class AbstractDocumentContent(metaclass=ABCMeta):
     def with_node_text(self: _AbstractDocumentContent, text: str) -> _AbstractDocumentContent:
         pass
 
+    @abstractmethod
+    def get_node_text_translation(self, lang: str) -> str:
+        pass
+
+    @property
+    @abstractmethod
+    def node_text_languages(self) -> Set[str]:
+        pass
+
+    @abstractmethod
+    def with_node_translation(self: _AbstractDocumentContent, lang: str, translation: str) -> _AbstractDocumentContent:
+        pass
+
+    @abstractmethod
+    def without_node_translation(self: _AbstractDocumentContent, lang: str) -> _AbstractDocumentContent:
+        pass
+
     @property
     @abstractmethod
     def original_node_text(self) -> str:
