@@ -1,5 +1,5 @@
 from copy import deepcopy
-from typing import Any, Callable, Dict, Iterable, Optional, Tuple, TypeVar
+from typing import Any, Callable, Dict, Iterable, Optional, Set, Tuple, TypeVar
 
 from intervaltree import IntervalTree
 
@@ -12,7 +12,7 @@ _TreeDocumentContent = TypeVar('_TreeDocumentContent', bound='TreeDocumentConten
 
 class TreeDocumentContent(TreeNode, AbstractTreeDocumentContent):
     __slots__ = (
-        '_node_type', '_text', '_markup', '_original_text', '_hidden',
+        '_node_type', '_text', '_text_translations', '_markup', '_original_text', '_hidden',
         '_text_references', '_concat_text_len', '_id2span', '_spans_tree'
     )
 
@@ -23,11 +23,13 @@ class TreeDocumentContent(TreeNode, AbstractTreeDocumentContent):
                  markup: Optional[Dict[str, Any]] = None,
                  original_text: Optional[str] = None,
                  hidden: bool = False,
-                 parent_func: Optional[Callable[[_TreeDocumentContent], _TreeDocumentContent]] = None):
+                 parent_func: Optional[Callable[[_TreeDocumentContent], _TreeDocumentContent]] = None,
+                 text_translations: Optional[Dict[str, str]] = None):
         TreeNode.__init__(self, node_id, nodes, parent_func)
 
         self._node_type = node_type
         self._text = text
+        self._text_translations = dict(text_translations) if text_translations is not None else {}
         self._markup = deepcopy(markup) if markup is not None else {}
         self._original_text = original_text
         self._hidden = hidden
@@ -66,11 +68,29 @@ class TreeDocumentContent(TreeNode, AbstractTreeDocumentContent):
     def node_text(self) -> str:
         return self._text
 
+    def get_node_text_translation(self, lang: str) -> str:
+        if lang not in self._text_translations:
+            raise ValueError(f"Tree node {self._id} doesn't contain translation to {lang}")
+        return self._text_translations[lang]
+
+    @property
+    def node_text_languages(self) -> Set[str]:
+        return set(self._text_translations.keys())
+
+    def with_node_translation(self: _TreeDocumentContent, lang: str, translation: str) -> _TreeDocumentContent:
+        return self._with_changes(text_translations={**self._text_translations, lang: translation})
+
+    def without_node_translation(self: _TreeDocumentContent, lang: str) -> _TreeDocumentContent:
+        copied = dict(self._text_translations)
+        if lang in copied:
+            del copied[lang]
+        return self._with_changes(text_translations=copied)
+
     @property
     def original_node_text(self) -> str:
         return self._original_text if self._original_text is not None else self._text
 
-    def with_node_text(self, text: str) -> _TreeDocumentContent:
+    def with_node_text(self: _TreeDocumentContent, text: str) -> _TreeDocumentContent:
         return self._with_changes(text=text)
 
     @property
@@ -91,7 +111,7 @@ class TreeDocumentContent(TreeNode, AbstractTreeDocumentContent):
         if isinstance(span, AbstractTalismanSpan):
             if span.node_id == self._id:
                 self._validate_talisman_span(span)
-                return (span, )
+                return (span,)
             return self._id2node[span.node_id].talisman_spans(span)
         self._validate_span(span)
         overlapped_node_ids = self._spans_tree.overlap(span.start_idx, span.end_idx)
@@ -138,6 +158,8 @@ class TreeDocumentContent(TreeNode, AbstractTreeDocumentContent):
             result['original_text'] = self._original_text
             result['text'] = self._text
 
+        result['text_translations'] = kwargs.get('text_translations', self._text_translations)
+
         result['markup'] = kwargs.get('markup', self._markup)
         result['hidden'] = kwargs.get('hidden', self._hidden)
 
@@ -148,3 +170,6 @@ class TreeDocumentContent(TreeNode, AbstractTreeDocumentContent):
             return NotImplemented
         return TreeNode.__eq__(self, other) and self._node_type == other._node_type and self._text == other._text and \
             self._markup == other._markup and self._original_text == other._original_text and self._hidden == other._hidden
+
+    def __hash__(self):
+        return hash((super().__hash__(), self._text, self._node_type, self._original_text, self._hidden))
