@@ -1,4 +1,4 @@
-from typing import Optional
+from typing import Dict, Optional
 
 from pydantic import BaseModel
 
@@ -9,6 +9,7 @@ from tdm.datamodel import TreeDocumentContent
 class NodeMetadata(BaseModel):
     node_type: NodeType
     original_text: Optional[str]
+    text_translations: Dict[str, str] = {}
     hidden: bool = False
 
     class Config:
@@ -22,5 +23,6 @@ class NodeMetadata(BaseModel):
         return cls.construct(
             node_type=node.type,
             original_text=node.original_node_text if node.original_node_text != node.node_text else None,
+            text_translations={lang: node.get_node_text_translation(lang) for lang in node.node_text_languages},
             hidden=node.is_hidden
         )
